﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Clocks; // to use class ClockTimer

namespace ClockGUI
{
    public partial class FormMain : Form
    {
        ////////////////////////////////////////
        private ClockTimer _timer;

        private DigitalClock _digitalClock;
        private AnalogClock _analogClock;
        ////////////////////////////////////////


        public FormMain()
        {
            InitializeComponent();

            ////////////////////////////////////////

            _timer = new ClockTimer(this);

            _digitalClock = new DigitalClock(this.labelDigitalClockTime);
            _analogClock = new AnalogClock(this.labelAnalogClockTime);


            ////////////////////////////////////////
        }

        private void buttonSetTime_Click(object sender, EventArgs e)
        {
            TimeSpan newTime;
            if (!TimeSpan.TryParse(textBoxNewTime.Text, out newTime))
                MessageBox.Show("Unknown time format", "Format error", 
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
            else
            {
                _timer.SetTime(newTime);
                MessageBox.Show("New time is " + newTime.ToString(), "Correct format", 
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void buttonStartTimer_Click(object sender, EventArgs e)
        {
            _timer.Start();
        }

        private void buttonStopTimer_Click(object sender, EventArgs e)
        {
            _timer.Stop();
        }

        private void checkBoxAttachDigitalClock_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox chb = (sender is CheckBox) ? sender as CheckBox : checkBoxAttachDigitalClock;

            if (chb.Checked)
                _timer.NotifyTick += _digitalClock.Update;
            else
                _timer.NotifyTick -= _digitalClock.Update;
        }

        private void checkBoxAttachAnalogClock_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox chb = (sender is CheckBox) ? sender as CheckBox : checkBoxAttachAnalogClock;

            if (chb.Checked)
                _timer.NotifyTick += _analogClock.Update;
            else
                _timer.NotifyTick -= _analogClock.Update;
        }
    }
}
