﻿using System;
using System.Timers;          // for "Timer"
using System.ComponentModel;  // for "ISynchronizeInvoke"
using System.Threading;       // for "ReaderWriterLock", but not for System.Threading.Timer ...

namespace Clocks
{
    public class ClockTimer //// OLD CODE // : Subject
    {
        public event EventHandler<TimeSpanEventArgs> NotifyTick;

        private readonly object _startAndStopRegionLock = new object();
        private readonly ReaderWriterLock _rwLock = new ReaderWriterLock();
        private readonly System.Timers.Timer _timer = new System.Timers.Timer();
        
        private TimeSpan _currentTime = TimeSpan.Zero; // default value 00:00:00

        public ClockTimer(ISynchronizeInvoke visualControl)
        {
            _timer.Interval = 1000; // 1000 msec == 1 sec
            _timer.Elapsed += _timer_Elapsed;

            // When SynchronizingObject is null, the method that handles the Elapsed
            // event is called on a thread from the system-thread pool [...]. 
            // When the Elapsed event is handled by a visual Windows Forms component,
            // such as a button, accessing the component through the system-thread pool
            // might result in an exception or just might not work. Avoid this effect by
            // setting SynchronizingObject to a Windows Forms component, which causes
            // the method that handles the Elapsed event to be called on the same thread
            // that the component was created on. 
            // [https://msdn.microsoft.com/it-it/library/system.timers.timer.synchronizingobject%28v=vs.110%29.aspx]

            if (visualControl != null)
                _timer.SynchronizingObject = visualControl;
        }

        public ClockTimer() : 
            this(null) // calling one-argument constructor
        { 
            // nothing to do, here
        }

        public void SetTime(TimeSpan newTime) 
        {
            _rwLock.AcquireWriterLock(Timeout.Infinite); // Timeout.Infinite == -1
            try { _currentTime = newTime; }
            finally { _rwLock.ReleaseWriterLock(); }
        }

        public TimeSpan GetTime() 
        {
            TimeSpan result;
            _rwLock.AcquireReaderLock(Timeout.Infinite);
            try { result = _currentTime; }
            finally { _rwLock.ReleaseReaderLock(); }
            return result;
        }

        public void Start()
        {
            lock (_startAndStopRegionLock)
            {  // critical read+write region
                if (!_timer.Enabled)
                    _timer.Start();
            }
        }

        public void Stop() 
        {
            lock (_startAndStopRegionLock) // critical r+w region
            {
                // critical read+write region
                if (_timer.Enabled)
                    _timer.Stop();
            }
        }

        private void _timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            Tick();
        }

        private void Tick()
        {
            _rwLock.AcquireWriterLock(Timeout.Infinite);
            try { _currentTime += new TimeSpan(0, 0, 1); } // increment: 1 sec
            finally { _rwLock.ReleaseWriterLock(); }
            EventHandler<TimeSpanEventArgs> tempTick = NotifyTick;
            if (tempTick != null)
                try { tempTick(this, new TimeSpanEventArgs(_currentTime)); }
                catch { }
           
            //// OLD CODE // Notify();
        }
    }
}
